"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// TODO: elliptic is only used here for secp256k1 opts. This can be replaced with a faster,
//       slimmer lib. 
const elliptic_1 = require("elliptic");
const cryptoRandom_1 = require("./cryptoRandom");
const errors_1 = require("../errors");
const keys_1 = require("../keys");
const sha2Hash_1 = require("./sha2Hash");
const hmacSha256_1 = require("./hmacSha256");
const aesCipher_1 = require("./aesCipher");
const ecurve = new elliptic_1.ec('secp256k1');
/**
* @ignore
*/
async function aes256CbcEncrypt(iv, key, plaintext) {
    const cipher = await aesCipher_1.createCipher();
    const result = await cipher.encrypt('aes-256-cbc', key, iv, plaintext);
    return result;
}
/**
* @ignore
*/
async function aes256CbcDecrypt(iv, key, ciphertext) {
    const cipher = await aesCipher_1.createCipher();
    const result = await cipher.decrypt('aes-256-cbc', key, iv, ciphertext);
    return result;
}
/**
* @ignore
*/
async function hmacSha256(key, content) {
    const hmacSha256 = await hmacSha256_1.createHmacSha256();
    return hmacSha256.digest(key, content);
}
/**
* @ignore
*/
function equalConstTime(b1, b2) {
    if (b1.length !== b2.length) {
        return false;
    }
    let res = 0;
    for (let i = 0; i < b1.length; i++) {
        res |= b1[i] ^ b2[i]; // jshint ignore:line
    }
    return res === 0;
}
/**
* @ignore
*/
async function sharedSecretToKeys(sharedSecret) {
    // generate mac and encryption key from shared secret
    const sha2Hash = await sha2Hash_1.createSha2Hash();
    const hashedSecret = await sha2Hash.digest(sharedSecret, 'sha512');
    return {
        encryptionKey: hashedSecret.slice(0, 32),
        hmacKey: hashedSecret.slice(32)
    };
}
/**
* @ignore
*/
function getHexFromBN(bnInput) {
    const hexOut = bnInput.toString('hex');
    if (hexOut.length === 64) {
        return hexOut;
    }
    else if (hexOut.length < 64) {
        // pad with leading zeros
        // the padStart function would require node 9
        const padding = '0'.repeat(64 - hexOut.length);
        return `${padding}${hexOut}`;
    }
    else {
        throw new Error('Generated a > 32-byte BN for encryption. Failing.');
    }
}
exports.getHexFromBN = getHexFromBN;
/**
 * Encrypt content to elliptic curve publicKey using ECIES
 * @param {String} publicKey - secp256k1 public key hex string
 * @param {String | Buffer} content - content to encrypt
 * @return {Object} Object containing (hex encoded):
 *  iv (initialization vector), cipherText (cipher text),
 *  mac (message authentication code), ephemeral public key
 *  wasString (boolean indicating with or not to return a buffer or string on decrypt)
 *
 * @private
 * @ignore
 */
async function encryptECIES(publicKey, content) {
    const isString = (typeof (content) === 'string');
    // always copy to buffer
    const plainText = content instanceof Buffer ? Buffer.from(content) : Buffer.from(content);
    const ecPK = ecurve.keyFromPublic(publicKey, 'hex').getPublic();
    const ephemeralSK = ecurve.genKeyPair();
    const ephemeralPK = ephemeralSK.getPublic();
    const sharedSecret = ephemeralSK.derive(ecPK);
    const sharedSecretHex = getHexFromBN(sharedSecret);
    const sharedKeys = await sharedSecretToKeys(Buffer.from(sharedSecretHex, 'hex'));
    const initializationVector = cryptoRandom_1.randomBytes(16);
    const cipherText = await aes256CbcEncrypt(initializationVector, sharedKeys.encryptionKey, plainText);
    const macData = Buffer.concat([initializationVector,
        Buffer.from(ephemeralPK.encode('array', true)),
        cipherText]);
    const mac = await hmacSha256(sharedKeys.hmacKey, macData);
    return {
        iv: initializationVector.toString('hex'),
        ephemeralPK: ephemeralPK.encode('hex', true),
        cipherText: cipherText.toString('hex'),
        mac: mac.toString('hex'),
        wasString: isString
    };
}
exports.encryptECIES = encryptECIES;
/**
 * Decrypt content encrypted using ECIES
 * @param {String} privateKey - secp256k1 private key hex string
 * @param {Object} cipherObject - object to decrypt, should contain:
 *  iv (initialization vector), cipherText (cipher text),
 *  mac (message authentication code), ephemeralPublicKey
 *  wasString (boolean indicating with or not to return a buffer or string on decrypt)
 * @return {Buffer} plaintext
 * @throws {FailedDecryptionError} if unable to decrypt
 * @private
 * @ignore
 */
async function decryptECIES(privateKey, cipherObject) {
    const ecSK = ecurve.keyFromPrivate(privateKey, 'hex');
    let ephemeralPK = null;
    try {
        ephemeralPK = ecurve.keyFromPublic(cipherObject.ephemeralPK, 'hex').getPublic();
    }
    catch (error) {
        throw new errors_1.FailedDecryptionError('Unable to get public key from cipher object. '
            + 'You might be trying to decrypt an unencrypted object.');
    }
    const sharedSecret = ecSK.derive(ephemeralPK);
    const sharedSecretBuffer = Buffer.from(getHexFromBN(sharedSecret), 'hex');
    const sharedKeys = await sharedSecretToKeys(sharedSecretBuffer);
    const ivBuffer = Buffer.from(cipherObject.iv, 'hex');
    const cipherTextBuffer = Buffer.from(cipherObject.cipherText, 'hex');
    const macData = Buffer.concat([ivBuffer,
        Buffer.from(ephemeralPK.encode('array', true)),
        cipherTextBuffer]);
    const actualMac = await hmacSha256(sharedKeys.hmacKey, macData);
    const expectedMac = Buffer.from(cipherObject.mac, 'hex');
    if (!equalConstTime(expectedMac, actualMac)) {
        throw new errors_1.FailedDecryptionError('Decryption failed: failure in MAC check');
    }
    const plainText = await aes256CbcDecrypt(ivBuffer, sharedKeys.encryptionKey, cipherTextBuffer);
    if (cipherObject.wasString) {
        return plainText.toString();
    }
    else {
        return plainText;
    }
}
exports.decryptECIES = decryptECIES;
/**
 * Sign content using ECDSA
 *
 * @param {String} privateKey - secp256k1 private key hex string
 * @param {Object} content - content to sign
 * @return {Object} contains:
 * signature - Hex encoded DER signature
 * public key - Hex encoded private string taken from privateKey
 * @private
 * @ignore
 */
async function signECDSA(privateKey, content) {
    const contentBuffer = content instanceof Buffer ? content : Buffer.from(content);
    const ecPrivate = ecurve.keyFromPrivate(privateKey, 'hex');
    const publicKey = keys_1.getPublicKeyFromPrivate(privateKey);
    const sha2Hash = await sha2Hash_1.createSha2Hash();
    const contentHash = await sha2Hash.digest(contentBuffer);
    const signature = ecPrivate.sign(contentHash);
    const signatureString = signature.toDER('hex');
    return {
        signature: signatureString,
        publicKey
    };
}
exports.signECDSA = signECDSA;
/**
* @ignore
*/
function getBuffer(content) {
    if (content instanceof Buffer)
        return content;
    else if (content instanceof ArrayBuffer)
        return Buffer.from(content);
    else
        return Buffer.from(content);
}
/**
 * Verify content using ECDSA
 * @param {String | Buffer} content - Content to verify was signed
 * @param {String} publicKey - secp256k1 private key hex string
 * @param {String} signature - Hex encoded DER signature
 * @return {Boolean} returns true when signature matches publickey + content, false if not
 * @private
 * @ignore
 */
async function verifyECDSA(content, publicKey, signature) {
    const contentBuffer = getBuffer(content);
    const ecPublic = ecurve.keyFromPublic(publicKey, 'hex');
    const sha2Hash = await sha2Hash_1.createSha2Hash();
    const contentHash = await sha2Hash.digest(contentBuffer);
    return ecPublic.verify(contentHash, signature);
}
exports.verifyECDSA = verifyECDSA;
//# sourceMappingURL=ec.js.map