"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cryptoUtils_1 = require("./cryptoUtils");
class NodeCryptoPbkdf2 {
    constructor(pbkdf2) {
        this.pbkdf2 = pbkdf2;
    }
    async derive(password, salt, iterations, keyLength, digest) {
        if (digest !== 'sha512' && digest !== 'sha256') {
            throw new Error(`Unsupported digest "${digest}" for Pbkdf2`);
        }
        return new Promise((resolve, reject) => {
            this.pbkdf2(password, salt, iterations, keyLength, digest, (error, result) => {
                if (error) {
                    reject(error);
                }
                resolve(result);
            });
        });
    }
}
class WebCryptoPbkdf2 {
    constructor(subtleCrypto) {
        this.subtleCrypto = subtleCrypto;
    }
    async derive(password, salt, iterations, keyLength, digest) {
        let algo;
        if (digest === 'sha256') {
            algo = 'SHA-256';
        }
        else if (digest === 'sha512') {
            algo = 'SHA-512';
        }
        else {
            throw new Error(`Unsupported Pbkdf2 digest algorithm "${digest}"`);
        }
        const passwordBytes = Buffer.from(password);
        const key = await this.subtleCrypto.importKey('raw', passwordBytes, 'PBKDF2', false, ['deriveBits']);
        const result = await this.subtleCrypto.deriveBits({
            name: 'PBKDF2', salt, iterations, hash: { name: algo }
        }, key, keyLength * 8);
        return Buffer.from(result);
    }
}
async function createPbkdf2() {
    const cryptoLib = await cryptoUtils_1.getCryptoLib();
    if (cryptoLib.name === 'subtleCrypto') {
        return new WebCryptoPbkdf2(cryptoLib.lib);
    }
    else {
        return new NodeCryptoPbkdf2(cryptoLib.lib.pbkdf2);
    }
}
exports.createPbkdf2 = createPbkdf2;
//# sourceMappingURL=pbkdf2.js.map